@@ -20,9 +20,11 @@ from imageio.core.functions import to_nbytes
 from imageio.testing import run_tests_if_main, get_test_dir, need_internet
 
 import imageio
+import imageio as iio
 from imageio import core
 from imageio.core import Request
 from imageio.core import get_remote_file, IS_PYPY
+from imageio.core.request import Mode, InitializationError
 
 
 try:
@@ -34,7 +36,7 @@ test_dir = get_test_dir()
 
 
 def test_fetching():
-    """ Test fetching of files """
+    """Test fetching of files"""
 
     need_internet()
 
@@ -124,14 +126,12 @@ def test_findlib2():
         os.mkdir(fi_dir)
     dirs, paths = core.findlib.generate_candidate_libs(["libfreeimage"], [fi_dir])
     # assert fi_dir in dirs -> Cannot test: lib may not exist
-    assert paths
 
     open(os.path.join(fi_dir, "notalib.test.so"), "wb")
 
     # Loading libs
     gllib = ctypes.util.find_library("GL")
     core.load_lib([gllib], [])
-    core.load_lib([], ["libfreeimage"], [fi_dir])
     # Fail
     raises(ValueError, core.load_lib, [], [])  # Nothing given
     raises(ValueError, core.load_lib, ["", None], [])  # Nothing given
@@ -140,7 +140,7 @@ def test_findlib2():
 
 
 def test_request():
-    """ Test request object """
+    """Test request object"""
 
     # Check uri-type, this is not a public property, so we test the private
     R = Request("http://example.com", "ri")
@@ -172,11 +172,17 @@ def test_request():
     R = Request("~/bar.zip/spam.png", "wi")
     assert R._uri_type == core.request.URI_ZIPPED
 
+    R = Request(b"x", "r")
+    assert R._mode == "r?"
+
+    R = Request("~/bar.zip/spam.png", "w")
+    assert R._mode == "w?"
+
     # Test failing inits
     raises(ValueError, Request, "/some/file", None)  # mode must be str
     raises(ValueError, Request, "/some/file", 3)  # mode must be str
     raises(ValueError, Request, "/some/file", "")  # mode must be len 2
-    raises(ValueError, Request, "/some/file", "r")  # mode must be len 2
+    # raises(ValueError, Request, "/some/file", "r")  # mode must be len 2
     raises(ValueError, Request, "/some/file", "rii")  # mode must be len 2
     raises(ValueError, Request, "/some/file", "xi")  # mode[0] must be in rw
     raises(ValueError, Request, "/some/file", "rx")  # mode[1] must be in iIvV?
@@ -388,6 +394,10 @@ def test_util_image():
     isinstance(im, np.ndarray)
     isinstance(im.meta, dict)
     assert str(im) == str(a)
+    # Preserve after copy
+    im2 = core.util.Image(im)
+    assert isinstance(im2, core.util.Image)
+    assert im2.meta == im.meta
     # Preserve after action
     im2 = im + 1
     assert isinstance(im2, core.util.Image)
@@ -457,7 +467,7 @@ def test_util_asarray():
 
 
 def test_util_progres_bar(sleep=0):
-    """ Test the progress bar """
+    """Test the progress bar"""
     # This test can also be run on itself to *see* the result
 
     # Progress bar
@@ -502,7 +512,7 @@ def test_util_progres_bar(sleep=0):
 
 
 def test_util_image_as_uint():
-    """ Tests the various type conversions when writing to uint"""
+    """Tests the various type conversions when writing to uint"""
     raises(ValueError, core.image_as_uint, 4)
     raises(ValueError, core.image_as_uint, "not an image")
     raises(ValueError, core.image_as_uint, np.array([0, 1]), bitdepth=13)
@@ -578,7 +588,7 @@ def test_util_has_has_module():
 
 
 def test_functions():
-    """ Test the user-facing API functions """
+    """Test the user-facing API functions"""
 
     # Test help(), it prints stuff, so we just check whether that goes ok
     imageio.help()  # should print overview
@@ -729,7 +739,7 @@ def test_memtest():
 
 
 def test_example_plugin():
-    """ Test the example plugin """
+    """Test the example plugin"""
 
     fname = os.path.join(test_dir, "out.png")
     r = Request("imageio:chelsea.png", "r?")
@@ -773,4 +783,105 @@ def test_imwrite_not_array_like():
         imageio.imwrite("foo.bmp", "asd")
 
 
+def test_legacy_empty_image():
+    with pytest.raises(RuntimeError):
+        with iio.imopen("foo.bmp", "wI", format="GIF-PIL") as file:
+            file.write([])
+
+
+def test_imopen_unsupported_iomode():
+    with pytest.raises(ValueError):
+        iio.imopen("", "unknown_iomode")
+
+
+def test_imopen_no_plugin_found(clear_plugins):
+    with pytest.raises(IOError):
+        iio.imopen("unknown.abcd", "r", search_legacy_only=False)
+
+
+@pytest.mark.parametrize("invalid_file", [".jpg"], indirect=["invalid_file"])
+def test_imopen_unregistered_plugin(clear_plugins, invalid_file):
+    with pytest.raises(ValueError):
+        iio.imopen(invalid_file, "r", plugin="unknown_plugin")
+
+
+def test_plugin_selection_failure(clear_plugins, monkeypatch):
+    class DummyPlugin:
+        def __init__(self, request):
+            raise InitializationError("Can not read anything")
+
+    monkeypatch.setattr(
+        iio.imopen, "_known_plugins", {"plugin1": DummyPlugin, "plugin2": DummyPlugin}
+    )
+
+    with pytest.raises(IOError):
+        iio.imopen("", "r", search_legacy_only=False)
+
+
+@pytest.mark.parametrize("invalid_file", [".jpg"], indirect=["invalid_file"])
+def test_plugin_selection_success(clear_plugins, monkeypatch, invalid_file):
+    class DummyPlugin:
+        def __init__(self, request):
+            """Can read anything"""
+
+    monkeypatch.setattr(iio.imopen, "_known_plugins", {"plugin1": DummyPlugin})
+
+    instance = iio.imopen(invalid_file, "r", search_legacy_only=False)
+
+    assert isinstance(instance, DummyPlugin)
+
+
+def test_legacy_object_image_writing():
+    with pytest.raises(ValueError):
+        iio.mimwrite("foo.gif", np.array([[0]], dtype=object))
+
+
+def test_imiter(image_files: Path):
+    # maybe it would be better to load the image without using imageio, e.g.
+    # numpy_im = np.load(image_files / "newtonscradle_rgb.npy")
+
+    full_image = iio.v3.imread(
+        image_files / "newtonscradle.gif", plugin="pillow", mode="RGB"
+    )
+
+    for idx, im in enumerate(
+        iio.v3.imiter(image_files / "newtonscradle.gif", plugin="pillow", mode="RGB")
+    ):
+        assert np.allclose(full_image[idx, ...], im)
+
+
+def test_request_mode_backwards_compatibility():
+    mode = Mode("ri")
+    assert mode == "ri"
+    assert mode[0] == "r"
+    assert mode[1] == "i"
+
+
+def test_faulty_legacy_mode_access():
+    mode = Mode("ri")
+    with pytest.raises(IndexError):
+        mode[3]  # has no third component
+
+
+def test_mvolread_out_of_bytes():
+    with pytest.raises(RuntimeError):
+        imageio.mvolread(
+            "https://github.com/imageio/imageio-binaries/blob/master/images/multipage_rgb.tif?raw=true",
+            memtest="1B",
+        )
+
+
+def test_invalid_explicit_plugin(clear_plugins, monkeypatch):
+    class DummyPlugin:
+        def __init__(self, request):
+            raise InitializationError("Can not read anything")
+
+    monkeypatch.setattr(
+        iio.imopen, "_known_plugins", {"plugin1": DummyPlugin, "plugin2": DummyPlugin}
+    )
+
+    with pytest.raises(IOError):
+        iio.imopen("", "r", plugin="plugin1")
+
+
 run_tests_if_main()
