@@ -55,7 +55,7 @@ fnamebase = os.path.join(test_dir, "test")
 
 
 def get_ref_im(colors, crop, isfloat):
-    """ Get reference image with
+    """Get reference image with
     * colors: 0, 1, 3, 4
     * cropping: 0-> none, 1-> crop, 2-> crop with non-contiguous data
     * float: False, True
@@ -88,7 +88,7 @@ def assert_close(im1, im2, tol=0.0):
 
 
 def test_get_ref_im():
-    """ A test for our function to get test images """
+    """A test for our function to get test images"""
 
     crop = 0
     for f in (False, True):
@@ -437,9 +437,6 @@ def test_animated_gif():
 
 def test_ico():
 
-    if os.getenv("TRAVIS", "") == "true" and sys.version_info >= (3, 4):
-        skip("Freeimage ico is unstable for this Travis build")
-
     for isfloat in (False, True):
         for crop in (0,):
             for colors in (1, 3, 4):
